Package: RCytoGPS
Version: 1.2.13
Date: 2026-07-06
Title: Using Cytogenetics Data in R
Authors@R: c(person(given = c("Kevin", "R."),
                      family = "Coombes",
                      role = c("aut", "cre"),
                      email = "krc@silicovore.com"),
               person(given = "Dwayne",
                      family = "Tally",
                      role = "aut"))
Description: Defines classes and methods to process text-based
  cytogenetics using the CytoGPS web site, then import the results
  into R for further analysis and graphing.
Depends: R (>= 4.4)
Suggests: knitr, rmarkdown, Mercator
Imports: methods, graphics, grDevices, rjson
VignetteBuilder: knitr
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
Author: Kevin R. Coombes [aut, cre],
  Dwayne Tally [aut]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: R-Forge
Repository/R-Forge/Project: thresher
Repository/R-Forge/Revision: 299
Repository/R-Forge/DateTimeStamp: 2026-07-06 17:43:31
Date/Publication: 2026-07-06 17:43:31
NeedsCompilation: no
Packaged: 2026-07-06 18:04:10 UTC; rforge

###
### NAMESPACE
###
import(methods)
importFrom("graphics", "image", "barplot", "abline", "layout",
           "legend", "lines", "mtext", "par", "title",
           "axis", "plot", "rect", "text", "box")
importFrom("grDevices", "palette", "col2rgb", "rgb")
importFrom("rjson", "fromJSON")

exportClasses("Chromosome", "CytobandData")
export("Chromosome", "CytobandData")
exportMethods("image", "barplot", "summary")

export("readLGF", "preclean")

### get idioGPS.csv and save it as cytobandLocations
### make these into Rdata files
idiocolors <- c(acen='#aa0000', gneg='white', 
                gpos100='black', gpos25='gray75',
                gpos50='gray50', gpos75='gray25',
                gvar='darkblue', stalk='lightblue')

cytobandLocations <- read.csv("idioGPS.csv", row.names=1)
cytobandLocations$Chromosome <- factor(cytobandLocations$Chromosome,
                                       levels = paste("chr", c(1:22,"X", "Y"),
                                                      sep=""))
cytobandLocations <- cytobandLocations[order(cytobandLocations$Chromosome,
                                             cytobandLocations$loc.start),]

save(cytobandLocations, idiocolors,
     file = file.path("..", "..", "data", "cytobandLocations.rda"))

library(tidyverse)
library(stringr)
x <- read.csv("startKaryotypes.csv") # original data
y <- read.delim("badStrings.txt") # list of abnormalities strings need to delete from z

z <- data.frame(x)#copy of original data
View(y)#double check what I need to get rid of

#relacing the strings of the abnormality
z[,4:5] <- lapply(z[,4:5], function(x) gsub(".ish","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/nonclonal w","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/nonclonal[1]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/nonclonal w/clonal abnormalities[2]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("[3, one is 4n]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/nonclonal[2]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("clonal","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/4n[1]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/4n[2]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/4n[3]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/4n[4]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("(D7S486+;D7Z1+,D7S486-)","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("46,XX[2]","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("(D13S319-,LAMP+)","",x, fixed = TRUE))
z[,4:5] <- lapply(z[,4:5], function(x) gsub("/46,XX[17]","",x, fixed = TRUE))

View(x)#viewing original data to be compared
View(z)#Viewing altered data to see if replacing worked

write.csv(z, file = "Altered.StartKaryotypes.csv")

## ----opts, echo=FALSE, results="hide"-----------------------------------------
knitr::opts_chunk$set(fig.width=8, fig.height=5)

## ----mycss, results="asis", echo=FALSE----------------------------------------
cat('
<style type="text/css">
.figure { text-align: center; }
.caption { font-weight: bold; }
</style>
')

## ----Setup--------------------------------------------------------------------
library(RCytoGPS)
data(cytoData)
colnames(cytoData)
bandData <- CytobandData(cytoData)
Acolumns <- c("A.Loss", "A.Gain", "A.Fusion")

## ----gb, fig.cap="Figure 1: Cytoband level data along the genome.", fig.width=14, fig.height=7----
opar <- par(mfrow=c(2,1))
barplot(bandData, "A.Loss", col = "forestgreen")
barplot(bandData, "B.Loss", col="orange")
par(opar)

## ----v.plot2Chrom, fig.width=8, fig.height=5,fig.cap = "Figure 2: Vertical stacked barplot of LGF frequencies on chromosome 2 for type A samples."----
image(bandData, what = Acolumns[1:3], chr = 2)

## ----h.plot2Chrom, fig.width=5, fig.height=8, fig.cap = "Figure 3: Horizontal stacked barplot of LGF frequencies on chromosome 2 for type A samples."----
image(bandData, Acolumns[1:3], chr = 2, horiz = TRUE)

## ----v.plot1Chrom, fig.width=8, fig.height=5, fig.cap = "Figure 4: Vertical stacked barplot of LGF frequencies on chromosome 5 for type A and B samples."----
image(bandData, what = list("A.Loss", "B.Loss"), chr = 5)


## ----h.plot1Chrom, fig.width=5, fig.height=8, fig.cap = "Figure 5: Horizontal stacked barplot of LGF frequencies on chromosome 5 for type A and B samples."----
image(bandData, what = list("A.Loss", "B.Loss"), chr = 5, horiz = TRUE)

## ----makeid, fig.width=10, fig.height=8, fig.cap="Figure 6: Idiogram for one data columm."----
image(bandData, what = Acolumns[1], chr = "all", pal = "orange")

## ----biid, fig.width=15, fig.height=12, fig.cap="Figure 7: Idiogram to contrast two data columms."----
image(bandData, what = Acolumns[1:2], chr = "all",
      pal=c("orange", "forestgreen"))

## ----biid3, fig.width=17, fig.height=17, fig.cap="Figure 8: Idiogram to contrast two data columms."----
image(bandData, what = Acolumns[1:2], chr = "all", nrows = 3,
      pal=c("orange", "forestgreen"))

## ----hbiid, fig.width=8, fig.height=10, fig.cap="Figure 9: Idiogram to contrast two data columms."----
image(bandData, what = Acolumns[1:2], chr = "all", 
      pal=c("orange", "forestgreen"), horiz = TRUE)

## ----stackid, fig.width=17, fig.height=15, fig.cap="Figure 10: Idiogram for many data columns."----
image(bandData, what = Acolumns[1:3], chr = "all", 
              pal=c("forestgreen", "orange", "purple"), nrows=3, horiz = FALSE)

## ----hstackid, fig.width=14, fig.height=9, fig.cap="Figure 11: Idiogram for many data columns."----
image(bandData, what = Acolumns, chr = "all", 
              pal=c("forestgreen", "orange", "purple"), nrows=3, horiz= TRUE)

## ----si-----------------------------------------------------------------------
sessionInfo()

## I think the point is to extract/construct identifiers to use as row names.
rnames <- function(lst, item) {
  f <- function(ll, inds) {
    if ((ii <- match(item, names(ll), FALSE)))
      list(inds=c(inds, ii), len=length(ll[[ii]]))
    else if (all(is.atomic(unlist(ll, FALSE))) || !is.list(ll))
      NULL
    else
      lapply(seq_along(ll), function(i) f(ll[[i]], inds=c(inds, i)))
  }
  unlist(f(lst, NULL))
}

## Make up sample names that include row number, padded with leading zeros
padnames <- function(prefix, values, ndigits = NULL) {
  M <- max(values)
  if (is.null(ndigits)) {
    ndigits <- trunc(log10(M))
  }
  padlength <- sapply(values, function(n) sum(n < 10^(1:ndigits)))
  pad <- sapply(padlength, function(p) paste(rep("0", p),collapse = ""))
  paste(prefix, pad, values, sep="")
}


extractOneLGF <- function(J) {
  bands <- J$iscn2016_bands # cytoband labels/names
  ## extract the status
  OUT <- J$output
  KY <- matrix(NA, ncol = 2, nrow = length(OUT))
  colnames(KY) <- c("Status", "Karyotype")
  for (I in 1:length(OUT)) {
    x <- OUT[[I]]
    kary <- x$karyotype
    stat <- x$status
    KY[I,] <- c(stat, kary)
  }
  KY <- as.data.frame(KY, stringsAsFactors = FALSE)
  rownames(KY) <- padnames("RN", 1:length(OUT))
  KY$Status  <- factor(KY$Status)

  ## extract the (sub)clone ids
  clone <- unlist(lapply(OUT, function(x){
    if((x$status) %in% c("Success", "Fixable grammar error and success")) {
      1:length(lapply(x$parsing_result, "[[", "loss_gain_fusion_computing"))
    }
  }))
  ## extract the binary-mapped LGF data
  lgf <- lapply(OUT, function(x) { # Get the loss data from parsing_result
    if((x$status) %in% c("Success", "Fixable grammar error and success")) {
      lgf <- lapply(x$parsing_result, "[[","loss_gain_fusion_computing")
    }
  })
  ## compute row-name IDS
  df.ID <- as.data.frame(matrix(rnames(lgf, "loss"), ncol = 4, byrow = TRUE))
  id <- df.ID$V1
  if(any(df.ID$V2 != clone)) {
    warning("Disagreement among clone IDs.")
  }

  fullID <- apply(df.ID[, 1:3], 1, paste, collapse=".")
  if (any(duplicated(fullID))) {
    warning("Duplicated identifiers should not happen.")
  }
  ## Extract the loss, gain, and fusion binary data separately
  loss1 <- sapply(lgf, function(x){
    lapply(x, "[[","loss")
  })
  df_Loss <- as.data.frame(matrix(unlist(loss1), ncol = 916, byrow = TRUE),
                           stringsAsFactors = FALSE)
  colnames(df_Loss) <- paste("Loss", bands, sep = "_")
  
  gain1 <- sapply(lgf, function(x){
    lapply(x, "[[","gain")
  })
  df_Gain <- as.data.frame(matrix(unlist(gain1), ncol = 916, byrow = TRUE),
                           stringsAsFactors = FALSE)
  colnames(df_Gain) <- paste("Gain", bands, sep = "_")
  fusion1 <- sapply(lgf, function(x){
    lapply(x, "[[","fusion")
  })
  df_Fusion <- as.data.frame(matrix(unlist(fusion1), ncol = 916, byrow = TRUE),
                             stringsAsFactors = FALSE)
  colnames(df_Fusion) <- paste("Fusion", bands, sep = "_")
  ## Put them back together
  df.lgf <- cbind(df_Loss, df_Gain, df_Fusion)
  rownames(df.lgf) <- fullID
  df.lgf$ID = id
  df.lgf$Clones = clone
  w <- which(colnames(df.lgf) == "ID")
  df.lgf <- df.lgf[, c(w:(w + 1), 1:(w-1))]  
  list(Status = KY, LGF = df.lgf)
}

if(FALSE) { # maybe later...
setClass("LGF",
         slots = c(
           raw = "list",             # of data frame, rows = clones, columns = LGF-cytobands
           frequency = "data.frame", # rows = cytobands, columns = Loss, Gain, Fusion
           source = "character",     # file names
           size = "numeric",         # number of clones in each file
           CL = "data.frame"         # cytoband locations
         ))
}

Idioformat <- function(df, CL){
  ## element name will be the same as karyotype
  ## first example loss.1, loss.2
  Loss <- df[, grepl("Loss", names(df))]
  tags <- sub("Loss_", "", colnames(Loss))
  Gain <- df[, grepl("Gain", names(df))]
  Fusion <- df[, grepl("Fusion", names(df))]
  temp <- data.frame(Loss = colMeans(Loss > 0),
                     Gain = colMeans(Gain > 0),
                     Fusion = colMeans(Fusion > 0))
  rownames(temp) <- tags
  list(Frequency = temp[rownames(CL),], N = nrow(df))
}

readLGF <- function(files = NULL, folder = NULL, verbose = TRUE) {
  ## Figure out which files we want to read
  if (is.null(folder)) {
    folder  <-  "."
  }
  if (is.null(files)) {
    files  <- list.files(folder, pattern = "*.json",
                         full.names = FALSE) # character vector , one file per entry
  }
  if (length(files) < 1) {
    stop("No JSON input files to read.")
  }
  if (verbose) {
    message("Reading ", length(files), " file(s) from '", folder, "'.\n")
  }
  ## make sure that, when done,  we leave the working directory in the same state that we found it.
  home <- getwd()
  on.exit(setwd(home))
  setwd(folder)

  ## It might get large -- not said by Davis Guggenheim
  myJSON <- lapply(files, function(x) fromJSON(file = x)) # a list, one element per JSON file
  raw  <- lapply(myJSON, extractOneLGF)
  names(raw) <- sub(".json", "", basename(files))
  ## Get the summary statistics
  ick <- lapply(raw, function(R) {
    Idioformat(R$LGF, CL = cytobandLocations)
  })
  bundle <- do.call(cbind, lapply(ick, function(F) F$Frequency))
  size <- sapply(ick, function(F) F$N)
  list(source = files, raw = raw, frequency = bundle, size = size, CL = cytobandLocations)
}

plot1Chrom <- function(DATA, columns,  chr, labels = columns,
                       pal = palette(), horiz = FALSE, axes=TRUE,
                       legend = FALSE, resn = NULL,
                       sigcolumn = NA, sigcut = 0.01, alpha = 63,
                       dlim = NULL) {
  ## check sigcolumn, sigcut, alpha
  if (!is.na(sigcolumn)) {
    if (length(sigcut) == 0) stop("You must supply at least one significance cutoff!")
    if (length (sigcut) != length(alpha)) stop("Lengths of 'sigcut' and 'alpha' do not match.")
    sigcut <- sort(sigcut)
    alpha <- sort(alpha)
  }
  ## check valid short chromosome name
  if ( !(chr %in% c(1:22, "X", "Y")) ) stop("Invalid chromosome number.")
  chrname <- paste("chr", chr, sep="")
  ## check valid stack height
  NC <- length(columns)
  if (NC < 1) stop("You need to supply the name of at least one data column.")
  if (NC > 10) stop("Unable to show more than ten data columns.")
  while(length(pal) < NC) pal <- c(pal, pal)
  ## check that all columns exist
  if ( !all(columns %in% colnames(DATA)) ) stop("Unrecognized column name.")
  ## get the labels ready
  if (is.null(labels)) labels <- rep("", NC)
  while (length(labels) < NC) labels <- c(labels, labels)
  opar <- par(c("new", "mai", "bg", "usr"))
  on.exit(par(opar))
  ## fake plot to white screen so we can use par(new=TRUE) in loop below
  par(bg = "white", mai = c(0,0,0,0))
  plot(0, 0, xaxt="n", yaxt="n", xlab="", ylab="", type="n", axes=FALSE)
  ## get the figure size in inches
  fin <- par("fin")
  ## create a "vertical resolution" near 200
  V0     <- c(25, 25, 15, 10, 17, 14, 13, 12, 11, 10)
  V1     <- c(75, 50, 30, 20, 33, 28, 25, 22, 20, 18)
  if (horiz) {
    vres <- fin[2]/100
    hres <- fin[1]/(V0[NC] + NC*V1[NC])
  } else {
    vres <- fin[2]/(V0[NC] + NC*V1[NC])
    hres <- fin[1]/100
  }

  dumbposn <- seq(1, 250000000, length=2500)
  CL <- cytobandLocations
  clap <- CL[CL$Chromosome == chrname,]
  segset <- DATA[DATA$Chromosome == chrname,]
  if (is.null(resn)) {
    resn <- max(max(segset[, columns]))
  }
  y <- rep(NA, length(dumbposn))
  for(J in 1:nrow(clap)) {
    y[clap[J, "loc.start"] <= dumbposn & 
      dumbposn <= clap[J, "loc.end"] ] <- as.numeric(clap[J, "Stain"])
  }
  ## data bars
  for (II in 1:length(columns)) {
    K <- NC - II + 1
    column <- columns[K]
    kcolors <- c(makeTransparent(pal[K], alpha), pal[K])
    vals <- NA * y
    shades <- NA * y
    for(J in 1:nrow(clap)) {
      vals[clap[J, "loc.start"] <= dumbposn & 
           dumbposn <= clap[J, "loc.end"] ] <- as.numeric(segset[J, column])
      if (!is.na(sigcolumn)) {
        shades[clap[J, "loc.start"] <= dumbposn & 
               dumbposn <= clap[J, "loc.end"] ] <-
                    kcolors[1 + sum(segset[J, sigcolumn] < sigcut)]
      } else {
        shades[clap[J, "loc.start"] <= dumbposn & 
               dumbposn <= clap[J, "loc.end"] ] <- pal[K]
      }
    }
    if (horiz) {
      par(new = TRUE,
          mai=c(vres, hres * (1 + V0[NC] + (K-1)*V1[NC]),
                10*vres, hres * (1 + (II-1)*V1[NC])))
      barplot(rev(vals), horiz = horiz, border=NA, col = rev(shades), 
              xlim=c(0, 1.05*resn), yaxs="i", ylim = dlim,
              space=0, axes=FALSE)
      if (axes) {
        axis(3) # on top
        U <- par("usr")
        W <- (U[1] + U[2])/2
        mtext(labels[K], at = W, side=3, line=2.5)
      }
    } else {
      par(new = TRUE,
          mai=c(vres * (1 + V0[NC] + (K-1)*V1[NC]), 10*hres,
                vres * (1 + (II-1)*V1[NC]), hres))
      barplot(vals, horiz = horiz, border=NA, col = shades,
              ylim=c(0, 1.05*resn), xaxs="i", ylab = labels[K], xlim = dlim,
              space=0, axes = axes)
    }
  }
  ## chromosome
  if (horiz) {
    par(new=TRUE,
        mai=c(vres, 2*hres, 10*vres, hres * (1 + NC*V1[NC])))
  } else {
    par(new=TRUE,
        mai=c(2*vres, 10*hres, vres * (1 + NC*V1[NC]), hres))
  }
  image(Chromosome(chr), mai=par("mai"), horiz = horiz)
  par(opar)
  invisible(DATA)
}

makeIdiogram <- function(DATA, colname, color, axes = TRUE, legend = FALSE,
                       sigcolumn = NA, sigcut = 0.01, alpha = 63) {
  opar <- par(c("mfrow", "mai", "usr", "bg"))
  on.exit(par(opar))

  resn  <- max(max(DATA[, colname]))

  par(mfrow=c(2,12), mai=c(0, 0.1, 1, 0.1), bg='white')
  for (I in c(1:22, "X", "Y")) {
    plot1Chrom(DATA, colname, chr = I, pal = color,  horiz=TRUE, axes = axes,
               resn = resn,
               sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
  }
  if (legend) {
    par(opar)
    par(mai=c(1, 1, 1, 1), usr = c(0,1, 0, 1))
    box(lty="blank")
    legend("bottom", colname, col = color, lwd = 5)
  }
}

stackIdiogram  <- function(DATA, columns, pal = palette(), nrows = 2,
                           horiz = FALSE, axes = TRUE, legend = FALSE,
                           sigcolumn = NA, sigcut = 0.01, alpha = 63) {
  if(!nrows %in% 1:4) {
    stop("Number of rows must be 1, 2, 3, or 4.")
  }
  opar <- par(c("mfrow", "mai", "usr"))
  on.exit(par(opar))

  if (horiz) { # horizontal layout of vertical plots
    L1 <- function() { par(mfrow = c(24, 1)) }
    L2 <- function() { par(mfrow = c(12, 2)) }
    L3 <- function() { par(mfrow = c(8, 3)) }
    L4 <- function() { par(mfrow = c(6, 4)) }
  } else { # vertical layout of horizontal plots
    L1 <- function() { par(mfrow = c(1, 24)) }
    L2 <- function() { par(mfrow = c(2, 12)) }
    L3 <- function() { par(mfrow = c(3, 8)) }
    L4 <- function() { par(mfrow = c(4, 6)) }
  }
  switch(nrows, L1(), L2(), L3(), L4())

  resn  <- max(max(DATA[, columns]))

  for (I in c(1:22, "X", "Y")) { # for each chromosome
    plot1Chrom(DATA, columns, chr = I, pal = pal, horiz = !horiz, axes = axes,
               resn = resn,
               sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
  }
  if (legend) {
    par(opar)
    par(mai=c(1, 1, 1, 1), usr = c(0,1, 0, 1), new=TRUE)
    box(lty = "blank") # weird graphics hack
    if (horiz) {
      legend("right", columns, col = pal, lwd = 5)
    } else {
      legend("bottom", columns, col = pal, lwd = 5)
    }
  }
}

library(RCytoGPS)
## simulate data
set.seed(53106)
CL <- RCytoGPS:::cytobandLocations
fake <- as.data.frame(matrix(runif(nrow(CL) * 10), ncol = 10))
fake$PV <- runif(nrow(fake), 0, 1)
CL <- cbind(CL, fake) # extra colnames V1, ..., V10
CD <- CytobandData(CL)

## clipping
image(CD, 7, list("V1", "V2"))
image(CD, 7, list("V1", "V2"), clip = TRUE)

RCytoGPS:::plot2Chrom(CD@DATA, "V1", "V2", chr = 7, horiz = TRUE, clip = TRUE)
RCytoGPS:::plot2Chrom(CD@DATA, "V1", "V2", chr = 7, horiz = FALSE, clip = TRUE)
image(Chromosome(7, maxbase = 1.6e8), horiz = TRUE)

library(RCytoGPS)

try( x <- Chromosome(0) )   # fail
try( x <- Chromosome("Z") ) # fail
try( x <- Chromosome(1:2) ) # fail
try (x <- Chromosome(c(1, "Z"))) # fail
try (x <- Chromosome(list(1, "Z"))) # fail

x <- Chromosome(11)
image(x)
image(x, showBandNames = TRUE)
image(x, horiz = TRUE)
image(x, horiz = TRUE, showBandNames = TRUE)

image(x, col="red") # silently ignores bad input?

library(RCytoGPS)

sf <- system.file("PreClean", package = "RCytoGPS")
dir(sf)

bad <- read.delim(file.path(sf, "badStrings.txt"), header=FALSE)
bad <- as.vector(as.matrix(bad))
input <- read.csv(file.path(sf, "startKaryotypes.csv"))
colnames(input)
output <- read.csv(file.path(sf, "Altered.StartKaryotypes.csv"))[, -1]

myclean <- preclean(input, 4:5, bad)

C4 <- summary(output[,4] == myclean[,4])
names(C4) <- gsub("NA's", "NAs", names(C4))
C4
C5 <- summary(output[,5] == myclean[,5])
names(C5) <- gsub("NA's", "NAs", names(C5))
C5

if(FALSE) {
  foo <- data.frame(IN = input[,4], OUT = output[,4], MY = myclean[,4])[output[,4] != myclean[,4],]
  write.csv(foo, file="foo.csv")
}

preclean <- function(x, targetColumns, dirt) {
  if (is.numeric(targetColumns)) {
    if (any(targetColumns < 1)) {
      stop("Only positive column numbers are allowed.\n")
    }
    if (any(targetColumns > dim(x)[2])) {
      nx <- deparse(substitute(x))
      stop("Column numbers exceed dimension of x (", nx, ").\n", sep="")
    }
    targetColumns <- colnames(x)[targetColumns]
  }
  if (!all(targetColumns %in% colnames(x))) {
    warning("Will skip unknown column names.\n")
  }
  for (target in targetColumns) {
    if (!(target %in% colnames(x))) {
      cat("\tSkipping '", target, "'\n", sep="", file=stderr())
    }
    vec <- x[,target]
    for (smudge in dirt) {
      vec <- gsub(smudge, "", vec, fixed = TRUE)
    }
    x[,target] <- vec
  }
  x
}

plot2Chrom <- function(DATA, leftcol, rightcol, chr,
                       pal = c("blue", "red"),
                       horiz = FALSE, axes = TRUE, legend = FALSE, resn = NULL,
                       sigcolumn = NA, sigcut = 0.01, alpha = 63, clip = FALSE) {
  ## check sigcolumn, sigcut, alpha
  if (!is.na(sigcolumn)) {
    if (length(sigcut) == 0) stop("You must supply at least one significance cutoff!")
    if (length (sigcut) != length(alpha)) stop("Lengths of 'sigcut' and 'alpha' do not match.")
    sigcut <- sort(sigcut)
    alpha <- sort(alpha)
  }
  ## check valid short chromosome name
  if ( !(chr %in% c(1:22, "X", "Y")) ) stop("Invalid chromosome number.")
  chrname <- paste("chr", chr, sep="")
  ## check valid stack height
  while(length(pal) < 2) pal <- c(pal, pal)
  ## check that all columns exist
  if ( !(leftcol %in% colnames(DATA)) )
    stop("Unrecognized (left) column name.")
  if ( !(rightcol %in% colnames(DATA)) )
    stop("Unrecognized (right) column name.")
  opar <- par(c("bg", "new", "mai", "usr"))
  on.exit(par(opar))
  par(bg = "white", mai=c(0,0,0,0))
  ## fake plot to white screen so we can use par(new=TRUE) in loop below
  plot(0, 0, xaxt="n", yaxt="n", xlab="", ylab="", type="n", axes=FALSE)
  ## get the figure size in inches
  fin  <- par("fin")
  ## set the resolution
  V0 <- 15
  V1 <- 45

  CL <- cytobandLocations
  clap <- CL[CL$Chromosome == chrname,]
  segset <- DATA[DATA$Chromosome == chrname,]
  MX <- max(segset$loc.end)
  topgun <- ifelse(clip, 1e6 * (1 + trunc(MX/1e6)), 250000000)
  dumbposn <- seq(1, topgun, length=2500)
  if (is.null(resn)) {
    resn <- max(max(segset[, c(leftcol, rightcol)]))
  }
  y <- left <- right <- dex <- rep(NA, length(dumbposn))
  for(J in 1:nrow(clap)) {
    y[clap[J, "loc.start"] <= dumbposn &
      dumbposn <= clap[J, "loc.end"] ] <- as.numeric(clap[J, "Stain"])
    left[clap[J, "loc.start"] <= dumbposn &
         dumbposn <= clap[J, "loc.end"] ] <- as.numeric(segset[J, leftcol])
    right[clap[J, "loc.start"] <= dumbposn &
          dumbposn <= clap[J, "loc.end"] ] <- as.numeric(segset[J, rightcol])
    if (!is.na(sigcolumn)) {
      dex[clap[J, "loc.start"] <= dumbposn &
          dumbposn <= clap[J, "loc.end"] ] <- 1 + sum(segset[J, sigcolumn] < sigcut)
    } else {
      dex[clap[J, "loc.start"] <= dumbposn &
             dumbposn <= clap[J, "loc.end"] ] <- 1 + length(sigcut)
    }
  }
  leftpal <- c(makeTransparent(pal[1], alpha), pal[1])
  rightpal <- c(makeTransparent(pal[2], alpha), pal[2])
  if(horiz) {
    vres <- fin[2]/100
    hres <- fin[1]/(V0 + 2*V1)

    ## right
    par(new = TRUE,
        mai=c(vres, (1 + V0 + V1)*hres, 15*vres, 2*hres))
    barplot(rev(right), horiz=TRUE, border=NA, col = rev(rightpal[dex]),
            xlim=c(0, 1.05*resn), yaxs="i",
            space=0, axes = FALSE)
    if (axes) {
      axis(3) # on top
      U <- par("usr")
      W <- (U[1] + U[2])/2
      mtext(rightcol, at = W, side=3, line=2.5)
    }
    ## chromosome in the middle
    par(new=TRUE,
        mai=c(vres, (V1 + 2)*hres, 15*vres, (V1 + 2)*hres))
    image(Chromosome(chr, maxbase = topgun), mai=par("mai"), horiz = TRUE)
    ## left, pointing backwards
    par(new = TRUE,
        mai=c(vres, 2*hres, 15*vres, (1 + V0 + V1)*hres))
    barplot(-rev(left), horiz=TRUE, border=NA, col = rev(leftpal[dex]),
            xlim=c(-1.05*resn, 0), yaxs="i",
            space=0, axes=FALSE)
    if (axes) {
      axis(3) # on top
      U <- par("usr")
      W <- (U[1] + U[2])/2
      mtext(leftcol, at = W, side=3, line=2.5)
    }
  } else {
    vres <- fin[2]/(V0 + 2*V1)
    hres <- fin[1]/100

    ## right goes on top, pointing up
    par(new = TRUE,
        mai=c((1 + V0 + V1)*vres, 10*hres, 2*vres, hres))
    barplot(right, border=NA, col = rightpal[dex],
            ylim=c(0, 1.05*resn), xaxs="i", ylab = rightcol,
            space=0, axes = axes)
    ## chromosome in the middle
    par(new=TRUE,
        mai=c((V1 + 2)*vres, 10*hres, (V1 + 2)*vres, hres))
    image(Chromosome(chr, maxbase = topgun), mai=par("mai"), horiz = horiz)
    ## left goes on the bottom, and points down
    par(new = TRUE,
        mai=c(2*vres, 10*hres, (1 + V0 + V1)*vres, hres))
    barplot(-(left), border=NA, col = leftpal[dex],
            ylim=c(-1.05*resn, 0), xaxs="i", ylab = leftcol,
            space=0, axes = axes)
  }
  par(opar)
  invisible(DATA)
}

biIdiogram  <- function(DATA, leftcol, rightcol,
                        pal = c("blue", "red"), nrows = 2,
                        horiz = FALSE, axes = TRUE, legend = FALSE,
                        sigcolumn = NA, sigcut = 0.01, alpha = 63) {
  if(!nrows %in% 1:4) {
    stop("Number of rows must be 1, 2, 3, or 4.")
  }
  opar <- par(c("mfrow", "mai", "usr"))
  on.exit(par(opar))

  if (horiz) { # horizontal layout of vertical plots
    L1 <- function() { par(mfrow = c(24, 1)) }
    L2 <- function() { par(mfrow = c(12, 2)) }
    L3 <- function() { par(mfrow = c(8, 3)) }
    L4 <- function() { par(mfrow = c(6, 4)) }
  } else { # vertical layout of horizontal plots
    L1 <- function() { par(mfrow = c(1, 24)) }
    L2 <- function() { par(mfrow = c(2, 12)) }
    L3 <- function() { par(mfrow = c(3, 8)) }
    L4 <- function() { par(mfrow = c(4, 6)) }
  }
  switch(nrows, L1(), L2(), L3(), L4())

  resn <- max(max(DATA[, c(leftcol, rightcol)]))
  
  for (I in c(1:22, "X", "Y")) { # for each chromosome
    plot2Chrom(DATA, leftcol, rightcol, I, pal, !horiz,
               axes = axes, resn = resn,
               sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
  }
  if (legend) {
    par(opar)
    par(mai=c(1, 1, 1, 1), usr = c(0,1, 0, 1))
    if (horiz) {
      legend("right", c(leftcol, rightcol), col = pal, lwd = 5)
    } else {
      legend("bottom", c(leftcol, rightcol), col = pal, lwd = 5)
    }
  }
}

library(RCytoGPS)
## simulate data
set.seed(53106)
CL <- RCytoGPS:::cytobandLocations
fake <- as.data.frame(matrix(runif(nrow(CL) * 10), ncol = 10))
fake$PV <- runif(nrow(fake), 0, 1)
CL <- cbind(CL, fake) # extra colnames V1, ..., V10
CD <- CytobandData(CL)

try( image(CD))     # fails, since you need 'chr'
try( image(CD, 3) ) # fails, since you also need 'what'

### windows(width=16, height=16)
## (plot1Chrom) one chromsome, stacks, vertical
image(CD, chr = 1, what = "V1", debug = TRUE)
image(CD, chr = 1, what = c("V1", "V2"), debug = TRUE)
image(CD, chr = 13, what = c("V1", "V2", "V3","V4", "V5", "V6"), debug = TRUE)
image(CD, chr = 6, what = paste("V", 1:10, sep=""), debug = TRUE)

## (plot1Chrom) one chromosome, stacks, horizontal
image(CD, 6, paste("V", 1:2, sep=""), horiz = TRUE, debug = TRUE)
image(CD, 6, paste("V", 1:2, sep=""), axes = FALSE, debug = TRUE)
image(CD, 6, paste("V", 1:2, sep=""), horiz = TRUE, axes = FALSE, debug = TRUE)

image(CD, 8, paste("V", 1:2, sep=""), horiz = FALSE,
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(127, 192))

image(CD, 8, paste("V", 1:2, sep=""), horiz = TRUE,
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(127, 192), debug = TRUE)

# combined (plot1Chrom)
opar <- par(mfrow=c(2,1))
image(CD, 2, c("V1", "V2", "V3"), debug = TRUE)
image(CD, 13, c("V4", "V5", "V6"), debug = TRUE)
par(opar)

## (makeIdiogram)
image(CD, "all", "V10", pal = "forestgreen", debug = TRUE) # no names  is better
image(CD, "all", "V10", pal = "forestgreen", debug = TRUE, axes = TRUE) # names
image(CD, "all", "V10", pal = "forestgreen", legend = TRUE, debug = TRUE)
image(CD, "all", "V10", pal = "forestgreen", debug = TRUE,
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(63, 155))

## (plot2Chrom) one chromosome, two-sided
image(CD, 3, list("V1", "V2"), debug = TRUE) # default axes = TRUE and horiz = FALSE
image(CD, 3, list("V1", "V2"), axes = FALSE, debug = FALSE)
image(CD, 3, list("V1", "V2"), horiz = TRUE, debug = TRUE)
image(CD, 3, list("V1", "V2"), horiz = TRUE, axes = FALSE, debug = TRUE) # still names?

image(CD, 8, list("V1", "V2"),
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(127, 192),
      debug = TRUE)
image(CD, 8, list("V1", "V2"), horiz = TRUE,
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(127, 192),
      debug = TRUE)

## (biIdiogram)
image(CD, chr = "all", list("V1", "V2"), horiz=TRUE, debug = TRUE) # default axes = FALSE
image(CD, chr = "all", list("V1", "V2"), horiz=FALSE, debug  = TRUE)
image(CD, chr = "all", list("V1", "V2"), horiz=TRUE, axes = TRUE, debug = TRUE)
image(CD, chr = "all", list("V1", "V2"), horiz=FALSE, axes = TRUE, debug  = TRUE)
image(CD, chr = "all", list("V1", "V2"), horiz=TRUE, nrows = 3)
image(CD, chr = "all", list("V1", "V2"), horiz=FALSE, nrows = 3)
image(CD, chr = "all", list("V1", "V2"), horiz=TRUE, nrows = 4)
image(CD, chr = "all", list("V1", "V2"), horiz=FALSE, nrows = 4)
image(CD, chr = "all", list("V1", "V2"), horiz=TRUE, nrows = 1)
image(CD, chr = "all", list("V1", "V2"), horiz=FALSE, nrows = 1)
## still need to test legend and sigcolumn
image(CD, chr = "all", list("V1", "V2"), horiz=TRUE, legend = TRUE, debug = TRUE)
image(CD, chr = "all", list("V1", "V2"), horiz=FALSE, legend = TRUE, debug  = TRUE)

image(CD, chr = "all", list("V1", "V2"),
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(127, 192),
      debug = TRUE)
image(CD, chr = "all", list("V1", "V2"), horiz = TRUE,
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(127, 192),
      debug = TRUE)

# stackIdiograms
mycolumns <- paste("V", 1:3, sep="")
image(CD, chr = "all", mycolumns, debug = TRUE)
image(CD, chr = "all", mycolumns, nrows = 3, debug = TRUE)
image(CD, chr = "all", mycolumns, nrows = 4, horiz=FALSE, debug = TRUE)
image(CD, chr = "all", mycolumns, nrows = 4, horiz=TRUE, debug = TRUE)
image(CD, chr = "all", mycolumns, nrows = 4, axes=TRUE, horiz=FALSE, debug = TRUE)
image(CD, chr = "all", mycolumns, nrows = 4, axes=TRUE, horiz=TRUE, debug = TRUE)
## still need to check axes, legend, and sigcolumn

image(CD, chr = "all", mycolumns, debug = TRUE, legend = TRUE) # no good place to put it
image(CD, chr = "all", mycolumns, horiz = TRUE, debug = TRUE, legend = TRUE) # no good place to put it


image(CD, chr = "all", mycolumns, horiz = TRUE,
      sigcolumn = "PV", sigcut = c(0.01, 0.05), alpha = c(63, 155),
      debug = TRUE)

## pass alpha on 0-255 scale
makeTransparent<-function(someColor, alpha=100) {
  newColor<-col2rgb(someColor)
  apply(newColor, 2, function(curcoldata){
    rgb(red=curcoldata[1], green=curcoldata[2], blue=curcoldata[3],
        alpha=alpha, maxColorValue=255)})
}

showGenome <- function(altcol = "#FED4C4") {
  N<- as.numeric(cytobandLocations$Chromosome) %% 2
  M <- cumsum(table(cytobandLocations$Chromosome))
  midpt <- (c(M) + c(0, M[-length(M)]))/2
  labs <- sub("chr", "", names(midpt))

  image(1:length(N), 1:1, as.matrix(N), col=c("white", altcol),
        xaxt="n", xlab="", yaxt="n", ylab="")
  text(midpt, rep(1, length(midpt)), labs, adj=0.5)
}


genomeBarplot <- function(V, col = "blue", altcol = "#FED4C4", ylab="Percent",
                          h = NULL, debug = FALSE) {
  ## get the figure size in inches
  fin = par("fin")
  ## define intenal "resolution" based on figure size
  HT0 <- 16
  HT1 <- 80
  vres <- fin[2]/(HT0 + HT1)
  hres <- fin[1]/1000
  opar <- par(c("new", "mai", "bg"))
  on.exit(par(opar))

#  par(bg = "white", mai=c(0.01, 0.5412, 0.1, 0.2772))
  par(bg = "white", mai=c(17*vres, 54*hres, 3*vres, 28*hres))
  if (debug) {
    print(par("fin"))
    print(par("mai"))
  }
  barplot(V, border=NA, space=0, ylab=ylab, col=col, yaxs="i", xaxt="n")
  if (!is.null(h)) { abline(h = h) }
#  par(new=TRUE, mai=c(0.39, 0.86, 0.01, 0.63))
  par(new=TRUE, mai=c(10*vres, 86*hres, 81*vres, 63*hres))
  if (debug) {
    print(par("fin"))
    print(par("mai"))
  }
  showGenome(altcol = altcol)
}

setClass("CytobandData",
         slots = c(DATA = "data.frame",
                   INFO = "data.frame"
                   ))

setValidity("CytobandData", function(object) {
  msg <- ""
  if (nrow(object@INFO) != ncol(object@DATA)) {
    msg <- "Mismatched sizes."
  } else if (!all(rownames(object@INFO) == colnames(object@DATA))) {
    msg <- "Mismatched names."
  }
  if (!all(c("Chromosome", "loc.start", "loc.end") %in% colnames(object@DATA))) {
    msg <- paste(msg, "Missing required data column.")
  }
  if (!all(c("Label", "Description") %in% colnames(object@INFO))) {
    msg <- paste(msg, "Missing required info column.")
  }
  ifelse(msg == "", TRUE, msg)
})

CytobandData <- function(data, info, genome = NULL) {
  if (missing(info)) {
    info <- data.frame(Label = colnames(data),
                       Description = colnames(data))
    rownames(info) <- colnames(data)
  }
  if (!is.null(genome)) {
    ginfo <- data.frame(Label = colnames(genome),
                        Description = colnames(genome))
    rownames(ginfo) <- colnames(genome)
    data <- cbind(data, genome)
    info <- rbind(info, genome)
  }
  new("CytobandData", DATA = data, INFO = info)
}

setMethod("summary", "CytobandData", function(object, ...) {
  cat("A object of the 'CytobandData' class.\n")
  summary(object@DATA)
})

setMethod("barplot", "CytobandData",
          function(height, what, col = "blue", altcol = "#FED4C4",
                   ylab = "Percent", hline = NULL,
                   xform = function(x) x, ...) {
  if (is.list(what)) what <- unlist(what)
  if ( length(what) != 1) stop("'what' must identify exactly one data column.")
  genomeBarplot(xform(height@DATA[ , what]),
                col = col, altcol = altcol, ylab = ylab, h=hline)
})

setMethod("image", "CytobandData", function(x, chr, what,
         pal = palette(), nrows = 2, labels = NULL,
         horiz = FALSE, axes = chr != "all", debug = FALSE, legend = FALSE,
         sigcolumn = NA, sigcut = 0.01, alpha = 63, clip = FALSE) {
  if (length(chr) != 1) {
    stop("Invalid chromosome value.")
  }
  if (is.list(what)) {
    if (length(what) != 2) {
      stop("'what' must be a list of length exactly two.")
    }
    if(chr == "all") {
      if(debug) cat("biIdiogram\n", file = stderr())
      biIdiogram(x@DATA, what[[1]], what[[2]],
                 pal = pal, nrows = nrows, horiz = horiz,
                 axes = axes, legend = legend,
                 sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
    } else {
      if(debug) cat("plot2Chrom\n", file = stderr())
      plot2Chrom(x@DATA, what[[1]], what[[2]],
                 chr = chr, pal = pal,
                 horiz = !horiz, # changed our minds about what "horiz" means
                 axes = axes, legend = legend,
                 sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha,
                 clip = clip)
    }
  } else { # now we must have chr equal to a character vector
    if (chr == "all") {
      if (length(what) == 1) {
        if(debug) cat("makeIdiogram\n", file = stderr())
        makeIdiogram(x@DATA, what, color = pal, axes = axes, legend = legend,
                     sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
      } else {
        if(debug) cat("stackIdiogram\n", file = stderr())
        stackIdiogram(x@DATA, what, pal = pal,
                      horiz = !horiz, # changed our minds about "horiz"
                      axes = axes, nrows = nrows, legend = legend,
                      sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
      }
    } else {
      if(debug) cat("plot1Chrom\n", file = stderr())
      plot1Chrom(x@DATA, what, chr = chr,
                 labels = labels, pal = pal, axes = axes,
                 horiz = !horiz, # changed our minds about what "horiz" means
                 legend = legend,
                 sigcolumn = sigcolumn, sigcut = sigcut, alpha = alpha)
    }
  }
  invisible(x)
})

library(RCytoGPS)

set.seed(53106)
CL <- RCytoGPS:::cytobandLocations
fake <- as.data.frame(matrix(runif(nrow(CL) * 10), ncol = 10))
CL <- cbind(CL, fake) # extra colnames V1, ..., V10
all(diff(as.numeric(CL$Chromosome)) >= 0) # check chromosome order

CD <- CytobandData(CL)
barplot(CD, what = "V10")
barplot(CD, what = list("V1")) # automagically converts list to vector

try( barplot(CD, what = c("V1", "V10")) )    # fail
try( barplot(CD, what = list("V1", "V10")) ) # fail

# confirm usability within preset par grid
opar <- par(mfrow = c(3,1))
barplot(CD, "V2", "green")
barplot(CD, "V3", "orange")
barplot(CD, "V5", "purple")
par(opar)

setClass("Chromosome",
         slots = c(
           name = "character",
           label = "character",
           grid = "numeric",
           range = "numeric",
           stain = "numeric")
)

Chromosome <- function(I, res = 2500, maxbase = 250000000) {
  if (length(I) > 1) stop("Can only handle one chromosome at a time.")
  if (!(I %in% c(1:22, "X", "Y"))) stop("Invalid chromosome name!")
  chrname <- paste('chr', I, sep='')
  label <- paste('Chr', I)
  dumbposn <- seq(1, maxbase, length=res)
  YY <- cytobandLocations[cytobandLocations$Chromosome == chrname,]
  y <- rep(NA, length(dumbposn))
  for(i in 1:nrow(YY)) {
    y[YY[i, "loc.start"] <= dumbposn & 
      dumbposn <= YY[i, "loc.end"] ] <- as.numeric(YY[i, "Stain"])
  }
  new("Chromosome",
      name = chrname,
      label = label,
      grid = dumbposn,
      range = c(min(YY$loc.start), max(YY$loc.end)),
      stain = y)
}

setMethod("image", signature = "Chromosome", function(x,
            horiz = FALSE, mai = NULL, showBandNames = FALSE, ...) {
  if (is.null(mai)) {
    if (horiz) {
      mai <- c(0, 0.1, 0.5, 0.05)
    } else {
      mai <- c(0.05, 0.6, 0.02, 0)
    }
  }
  opar <- par(mai=mai)
  on.exit(par(opar))
  if (horiz) {
    pts <- max(x@grid) - x@range
    image(1:1, x@grid, matrix(rev(x@stain), nrow=1), col=idiocolors, bty='n',
          xlab='', ylab='', xaxt='n', yaxt='n', zlim=c(1, 8))
    title(main=x@label, adj=0, xpd = NA) # clip to device
    rect(0.6, pts[1], 1.4, pts[2])
    if (showBandNames) {
      foo <- findLabels(x@name)
      text(1, max(x@grid) - foo$midpt, rownames(foo), col=foo$labCol)
    }
  } else {
    pts <- x@range
    image(x@grid, 1:1, matrix(x@stain, ncol=1), col=idiocolors, bty='n',
          xlab='', ylab='', xaxt='n', yaxt='n', zlim=c(1, 8),
          cex=0.8)
    mtext(x@label, side=2, at=1, las=2, line=0.5)
    rect(pts[1], 0.6, pts[2], 1.4)
    if (showBandNames) {
      foo <- findLabels(x@name)
      text(foo$midpt, 1, rownames(foo), col=foo$labCol, srt=90)
    }
  }
  invisible(x)
})

idioLabelColors <- c(acen = "white", gneg = "black", gpos100 = "white", gpos25 = "black",
                     gpos50 = "black", gpos75 = "white", gvar = "white", stalk = "black")

findLabels <- function(chrom) {
  if (length(chrom) > 1) {
    warning("More than two chromosomes supplied; only using the first one")
    chrom <- chrom[1]
  }
  if (!grepl("^chr", chrom)) { # add the prefix
    chrom <- paste("chr", chrom, sep = "")
  }
  X <- cytobandLocations[cytobandLocations$Chromosome == chrom,]
  X$midpt <- (X$loc.start + X$loc.end)/2
  X$labCol <- idioLabelColors[X$Stain]
  X
}

library(RCytoGPS)

sf <- system.file("Examples/JSONfiles", package = "RCytoGPS")
dir(sf)

## Read one explicitly named JSON file
temp <- readLGF("CytoGPS_Result1.json", folder = sf, verbose = FALSE)
class(temp)
names(temp)
temp$source
class(temp$raw)
length(temp$raw)
names(temp$raw)
temp$raw[[1]]$Status
summary(temp$frequency)
temp$size

## Read all (well, two) JSON files fom a folder
temp <- readLGF(folder = sf, verbose = FALSE)
class(temp)
names(temp)
temp$source
class(temp$raw)
length(temp$raw)
names(temp$raw)
temp$raw[[1]]$Status
temp$raw[[2]]$Status
summary(temp$frequency)
temp$size

## move about
home  <- getwd()
setwd(sf)
temp <- readLGF( "CytoGPS_Result1.json")
temp <- readLGF( "CytoGPS_Result1.json", folder = ".")
setwd("..")
temp <- readLGF(folder = "JSONfiles")
setwd(home)

---
title: "RCytoGPS Gallery"
author: "Dwayne Tally, Zachary B. Abrams, and Kevin R. Coombes"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_document:
    theme: journal
    highlight: kate
  toc: true
vignette: >
  %\VignetteIndexEntry{RCytoGPS Gallery}
  %\VignetteKeywords{OOMPA,CytoGPS,karyotypes,idiograms,graphics}
  %\VignetteDepends{RCytoGPS}
  %\VignettePackage{RCytoGPS}
  %\VignetteEngine{knitr::rmarkdown}
---


```{r opts, echo=FALSE, results="hide"}
knitr::opts_chunk$set(fig.width=8, fig.height=5)
```
```{r mycss, results="asis", echo=FALSE}
cat('
<style type="text/css">
.figure { text-align: center; }
.caption { font-weight: bold; }
</style>
')
```
# Introduction
In this vignette, we focus on the different visualizations you can generate by
changing the arguments for the <tt>image</tt> and <tt>barplot</tt> methods. The main arguments
to <tt>image</tt>, when using a <tt>CytobandData</tt> object from the package are

* <tt>what</tt>: A list or vector indicating which data columns to show.
* <tt>chr</tt>: The chromosome to plot, which can take on the value <tt>all<tt> to show the whole genome

The <tt>barplot</tt> method only takes the <tt>what</tt> argument, which in this case must be a
length-one character vector.

To produce the figures in this gallery, we will use a dataset that comes with the
package, already processed,
```{r Setup}
library(RCytoGPS)
data(cytoData)
colnames(cytoData)
bandData <- CytobandData(cytoData)
Acolumns <- c("A.Loss", "A.Gain", "A.Fusion")
```
The <tt>banddata</tt> object includes data at the resolution of a cytoband for three sets
of samples (denoted A, B, and C), describing the frequency of loss, gain, and fusion
events for each set.

# Plotting Cytoband Data Along the Genome
The first visual is a barplot to show the frequencies of cytogenetic events mapped to the
associated chromosome. The main parameters for `barplot` are (1) the <tt>CytobandData</tt>
object containing the dataset, (2) the specific data column that  you want to graph, and
(3) the color of the graph.
```{r gb, fig.cap="Figure 1: Cytoband level data along the genome.", fig.width=14, fig.height=7}
opar <- par(mfrow=c(2,1))
barplot(bandData, "A.Loss", col = "forestgreen")
barplot(bandData, "B.Loss", col="orange")
par(opar)
```

# Single Chromosome Plots
Plots of the data along one chromosome can be oriented either horizontally or vertically.
By (our) definition, "horizontal" and "vertical" refer to the orientation of the chromosome
with its cytoband shading. Unless specified, the <tt>horiz</tt> argument is set to
<tt>FALSE</tt> by default.

## Plotting Cytoband-Level Data Along One Chromosome
The "vertical stacked chromosome" allows you to see a vertical barplot and its associated
chromosome on the y axis, this allows you to see multiple cytogenetic events on a specified
chromosome. The parameters to produce this image are the dataset, the cytogenetic event, and
a specific chromosome.

```{r v.plot2Chrom, fig.width=8, fig.height=5,fig.cap = "Figure 2: Vertical stacked barplot of LGF frequencies on chromosome 2 for type A samples."}
image(bandData, what = Acolumns[1:3], chr = 2)
```
The next visual is nearly identical to "vertical stack chromosome" with the exception that the visual is set along the x-axis instead of the y axis.

```{r h.plot2Chrom, fig.width=5, fig.height=8, fig.cap = "Figure 3: Horizontal stacked barplot of LGF frequencies on chromosome 2 for type A samples."}
image(bandData, Acolumns[1:3], chr = 2, horiz = TRUE)
```

## Plotting Cytoband-Level Data Along Both Sides of One Chromosome
This set of visuals allows the user to directly compare two sets of cytogenetic events
on a specified chromosome on the y-axis.
```{r v.plot1Chrom, fig.width=8, fig.height=5, fig.cap = "Figure 4: Vertical stacked barplot of LGF frequencies on chromosome 5 for type A and B samples."}
image(bandData, what = list("A.Loss", "B.Loss"), chr = 5)

```
The next visual is similar to the previous visual except that the orientation is switched
from vertical to horizontal.
```{r h.plot1Chrom, fig.width=5, fig.height=8, fig.cap = "Figure 5: Horizontal stacked barplot of LGF frequencies on chromosome 5 for type A and B samples."}
image(bandData, what = list("A.Loss", "B.Loss"), chr = 5, horiz = TRUE)
```

# Idiograms
We can also produce "idiograms" that show the data for all chromosomes at once rather than
just one chromosome at a time.

## One Data Column
The simplest idiogram visualizes a single data colum of cytogenetic events for all
chromosomes. The parameters are similar to the previous visuals with the exception
that <tt>chr</tt> is set to `all` and we describe the colors using the argument
<tt>pal</tt> (for pallete).
```{r makeid, fig.width=10, fig.height=8, fig.cap="Figure 6: Idiogram for one data columm."}
image(bandData, what = Acolumns[1], chr = "all", pal = "orange")
```

## Contrasting Two Data Columns
This image allows you to do a comparison between two cytogenetic events on all the chromosomes.
```{r biid, fig.width=15, fig.height=12, fig.cap="Figure 7: Idiogram to contrast two data columms."}
image(bandData, what = Acolumns[1:2], chr = "all",
      pal=c("orange", "forestgreen"))
```
The next visual is the same as the previous with the exception that the user can dictate
how many total rows they want to display. (The default is 2; permitted values range from
1 to 4.)
```{r biid3, fig.width=17, fig.height=17, fig.cap="Figure 8: Idiogram to contrast two data columms."}
image(bandData, what = Acolumns[1:2], chr = "all", nrows = 3,
      pal=c("orange", "forestgreen"))
```
This visual is the same as before except that we have switched from vertical to horizontal.
```{r hbiid, fig.width=8, fig.height=10, fig.cap="Figure 9: Idiogram to contrast two data columms."}
image(bandData, what = Acolumns[1:2], chr = "all", 
      pal=c("orange", "forestgreen"), horiz = TRUE)
```

## Many Data Columns
The <tt>image</tt> method also  allows you to graph more than two (up to ten) data columns.
```{r stackid, fig.width=17, fig.height=15, fig.cap="Figure 10: Idiogram for many data columns."}
image(bandData, what = Acolumns[1:3], chr = "all", 
              pal=c("forestgreen", "orange", "purple"), nrows=3, horiz = FALSE)
```
Lastly, this image is the same as the previous with the exception of the graphs being plotted horizontally.
```{r hstackid, fig.width=14, fig.height=9, fig.cap="Figure 11: Idiogram for many data columns."}
image(bandData, what = Acolumns, chr = "all", 
              pal=c("forestgreen", "orange", "purple"), nrows=3, horiz= TRUE)
```

# Appendix
```{r si}
sessionInfo()
```

---
title: "RCytoGPS: Working With LGF-Models of Karyotype in R"
author: "Dwayne Tally, Zachary B. Abrams, and Kevin R. Coombes"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_document:
    theme: journal
    highlight: kate
vignette: >
  %\VignetteIndexEntry{Introduction to RCytoGPS}
  %\VignetteKeywords{OOMPA,CytoGPS,karyotypes,idiograms,graphics}
  %\VignetteDepends{RCytoGPS}
  %\VignettePackage{RCytoGPS}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r opts, echo=FALSE}
knitr::opts_chunk$set(fig.width=8, fig.height=5)
```
```{r mycss, results="asis", echo=FALSE}
cat('
<style type="text/css">
.figure { text-align: center; }
.caption { font-weight: bold; }
</style>
')
```
# Introduction
Modern biological experiments are increasingly producing interesting binary matrices.
These may represent the presence or absence of specific gene mutations, copy number
variants, microRNAs, or other molecular or clinical phenomena. We recently developed
a tool, CytoGPS [^Abrams and colleagues], that converts conventional karyotypes from
the standard text-based notation (the International Standard for Human
Cytogenetic/Cytogenomic Nomenclature; ISCN) into a binary vector with three bits
(loss, gain, or fusion) per cytoband, which we call the "LGF model".

The CytoGPS tool is available at the web site http://cytogps.org, where the LGF results
of processing karyotype data are returned in JSON format. To complement the web site,
we have developed <tt>RCytoGPS</tt>, an R package to  extract, format, and visualize
genetic data at the resolution of cytobands. <tt>RCytoGPS</tt> can parse any JSON file
(or set of files) produced by CytoGPS.org.

## Setup
In order to extract LGF data from JSON files, you must first load the package.
```{r Setup}
library(RCytoGPS)
```

# Extracting JSON data and formatting to LGF model
We have included a pair of JSON files produced at CytoGPS.org as examples in the package.
These are found in the following directory:
```{r sysfile}
wd <-  system.file("Examples/JSONfiles", package = "RCytoGPS")
dir(wd)
```
The two text files contain the inputs that were uploaded to the web site; the two
JSON files contain the outputs. You can specify the files and the folder that you
want to read. The simplest application is to omit the files variable and read all
filed in teh specified folder (which defaults to the current working directory).
```{r readLGF}
temp <- readLGF(folder = wd)
rm(wd)
```
The return value is a list of five elements.
```{r peekTemp}
class(temp)
names(temp)
```
The <tt>source</tt> element documents which JSON file(s) were read.
```{r src}
temp$source
```
The <tt>size</tt> element lists the number of rows returned from each file; each
row represents a distinct clone.
```{r size}
temp$size
```
The <tt>CL</tt> element is a data frame describing the chromosomal locations of
each cytoband.
```{r CL}
summary(temp$CL)
```
The <tt>raw</tt> element is itself a list, containing the binary LGF data for
each JSON file processed. Each file produces a "Status" output along with the LGF
data. The Status includes both the input karyotype (in ISCN format) and an indicator of
whether CytoGPS could successfully process it.  In this example, the first karyotype
contained an error. As a result, the LGF component does not contain any rows derived
from that karyotype.  It does, however. contain three rows derived from the second
karyotype, since the "forward slashes" separate the decriptions of three different
clones that were detected in that sample.
```{r raw}
names(temp$raw)
R <- temp$raw[[2]]
names(R)
R$Status
dim(R$LGF)
rownames(R$LGF)
rm(R)
```
Finally, the <tt>frequency</tt> element contains summary data from each file read.
These summaries consist of the frequencies of loss, gain, and fusion events. Each row
of this data frame represents a cytoband. There are three columns from each JSON file,
one each for loss, gain, and fusion
```{r freqelt}
F <- temp$frequency
class(F)
dim(F)
colnames(F)
```

# Extracting the cytoband locations, and the frequency data
In order to be able to work with the cytoband-level frequency data, we must combine
it with the cytoband location data. Here we assemble them into a single data frame.
```{r cytoData}
cytoData <- data.frame(temp[["CL"]], temp[["frequency"]])
```

# Turning CytoData into an S4 Object
Next, we transfrom the CytoData data frame into an S4 object using the function
<tt.cytobandData</tt>. The newly acquired  object will then be used to generatie plots
and will be available for further analyses. 
```{r Turning CytoData into an S4 Object}
bandData <- CytobandData(cytoData)
```
## Generating Graphs

# Plotting Cytoband Data Along the Genome
The first graphs (using <tt>barplot</tt> ]) summarizes the frequency data from one data
column  along the genome. This provides a broad overview of the changes, and can be used
to visually contrast the locations of changes in different data sets. Here we use barplot
twice, showing losses and gains from the first file. 
```{r gb, fig.cap="Cytoband level data along the genome.", fig.width=14, fig.height=7}
opar <- par(mfrow=c(2,1))
barplot(bandData, what = "CytoGPS_Result1.Loss", col = "forestgreen")
barplot(bandData, what = "CytoGPS_Result1.Gain", col = "orange")
par(opar)
```

# Plotting Cytoband-Level Data Along One Chromosome
The next graph allows you to simultaneously compare multiple cytogenetic events one
chromosome at a time. 
```{r v.plot2Chrom, fig.width=8, fig.height=5,fig.cap = "Vertical stacked barplot of LGF frequencies on chromosome 3 for type 1 samples."}
datacolumns <- names(temp[["frequency"]])
datacolumns
image(bandData, what = datacolumns[1:3], chr = 2, labels = TRUE)
```
By adding the parameter <tt>horix=TRUE</tt>, you can rotate this graph 90 degrees. For
more details about the parameters of the <tt>image</tt> method, see the manual pages
and the "gallery" vignette.

# Idiograms
We can assemble all of the single-chromosome plots into a single "idiogram" graph that
shows all chromosomes at once.


## One Data Column
The purpose of this graph is to visualize the chromosomes as well as a barplot of the
cytogenetic abnormalities in orderto observe and possibly identify patterns.
```{r makeid, fig.width=10, fig.height=8, fig.cap="Idiogram for one data columm."}
image(bandData, what = datacolumns[1], chr = "all", pal = "orange")
```

## More Data Columns
This graph allows the user to compare and contrast two or more cytogenetic events
simultaneously. Here we show loss (orange), gain (green), and fusion (purple) events
from the Type 1 samples.
```{r biid, fig.width=10, fig.height=8, fig.cap="Idiogram to contrast two data columms."}
image(bandData, what = datacolumns[1:3], chr = "all", 
      pal=c("orange", "forestgreen", "purple"), horiz=TRUE)
```

# Gallery
To see all possible visuals please go to our gallery for images.

# Appendix
```{r}
sessionInfo()

```

---
title: "RCytoGPS: Working With LGF-Models of Karyotype in R"
author: "Dwayne Tally, Zachary B. Abrams, and Kevin R. Coombes"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_document:
    theme: journal
    highlight: kate
vignette: >
  %\VignetteIndexEntry{Introduction to RCytoGPS}
  %\VignetteKeywords{OOMPA,CytoGPS,karyotypes,idiograms,graphics}
  %\VignetteDepends{RCytoGPS}
  %\VignettePackage{RCytoGPS}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r opts, echo=FALSE}
knitr::opts_chunk$set(fig.width=8, fig.height=5)
```
```{r mycss, results="asis", echo=FALSE}
cat('
<style type="text/css">
.figure { text-align: center; }
.caption { font-weight: bold; }
</style>
')
```
# Introduction
Modern biological experiments are increasingly producing interesting binary matrices.
These may represent the presence or absence of specific gene mutations, copy number
variants, microRNAs, or other molecular or clinical phenomena. We recently developed
a tool, CytoGPS [^Abrams and colleagues], that converts conventional karyotypes from
the standard text-based notation (the International Standard for Human
Cytogenetic/Cytogenomic Nomenclature; ISCN) into a binary vector with three bits
(loss, gain, or fusion) per cytoband, which we call the "LGF model".

The CytoGPS tool is available at the web site 
https://cytogps-app.mangograss-a1cf8a6c.westus2.azurecontainerapps.io/,
where the LGF results
of processing karyotype data are returned in JSON format. To complement the web site,
we have developed <tt>RCytoGPS</tt>, an R package to  extract, format, and visualize
genetic data at the resolution of cytobands. <tt>RCytoGPS</tt> can parse any JSON file
(or set of files) produced by CytoGPS.org.

## Setup
In order to extract LGF data from JSON files, you must first load the package.
```{r Setup}
library(RCytoGPS)
```

# Extracting JSON data and formatting to LGF model
We have included a pair of JSON files produced at CytoGPS.org as examples in the package.
These are found in the following directory:
```{r sysfile}
wd <-  system.file("Examples/JSONfiles", package = "RCytoGPS")
dir(wd)
```
The two text files contain the inputs that were uploaded to the web site; the two
JSON files contain the outputs. You can specify the files and the folder that you
want to read. The simplest application is to omit the files variable and read all
filed in teh specified folder (which defaults to the current working directory).
```{r readLGF}
temp <- readLGF(folder = wd)
rm(wd)
```
The return value is a list of five elements.
```{r peekTemp}
class(temp)
names(temp)
```
The <tt>source</tt> element documents which JSON file(s) were read.
```{r src}
temp$source
```
The <tt>size</tt> element lists the number of rows returned from each file; each
row represents a distinct clone.
```{r size}
temp$size
```
The <tt>CL</tt> element is a data frame describing the chromosomal locations of
each cytoband.
```{r CL}
summary(temp$CL)
```
The <tt>raw</tt> element is itself a list, containing the binary LGF data for
each JSON file processed. Each file produces a "Status" output along with the LGF
data. The Status includes both the input karyotype (in ISCN format) and an indicator of
whether CytoGPS could successfully process it.  In this example, the first karyotype
contained an error. As a result, the LGF component does not contain any rows derived
from that karyotype.  It does, however. contain three rows derived from the second
karyotype, since the "forward slashes" separate the decriptions of three different
clones that were detected in that sample.
```{r raw}
names(temp$raw)
R <- temp$raw[[2]]
names(R)
R$Status
dim(R$LGF)
rownames(R$LGF)
rm(R)
```
Finally, the <tt>frequency</tt> element contains summary data from each file read.
These summaries consist of the frequencies of loss, gain, and fusion events. Each row
of this data frame represents a cytoband. There are three columns from each JSON file,
one each for loss, gain, and fusion
```{r freqelt}
F <- temp$frequency
class(F)
dim(F)
colnames(F)
```

# Extracting the cytoband locations, and the frequency data
In order to be able to work with the cytoband-level frequency data, we must combine
it with the cytoband location data. Here we assemble them into a single data frame.
```{r cytoData}
cytoData <- data.frame(temp[["CL"]], temp[["frequency"]])
```

# Turning CytoData into an S4 Object
Next, we transfrom the CytoData data frame into an S4 object using the function
<tt.cytobandData</tt>. The newly acquired  object will then be used to generatie plots
and will be available for further analyses. 
```{r Turning CytoData into an S4 Object}
bandData <- CytobandData(cytoData)
```
## Generating Graphs

# Plotting Cytoband Data Along the Genome
The first graphs (using <tt>barplot</tt> ]) summarizes the frequency data from one data
column  along the genome. This provides a broad overview of the changes, and can be used
to visually contrast the locations of changes in different data sets. Here we use barplot
twice, showing losses and gains from the first file. 
```{r gb, fig.cap="Cytoband level data along the genome.", fig.width=14, fig.height=7}
opar <- par(mfrow=c(2,1))
barplot(bandData, what = "CytoGPS_Result1.Loss", col = "forestgreen")
barplot(bandData, what = "CytoGPS_Result1.Gain", col = "orange")
par(opar)
```

# Plotting Cytoband-Level Data Along One Chromosome
The next graph allows you to simultaneously compare multiple cytogenetic events one
chromosome at a time. 
```{r v.plot2Chrom, fig.width=8, fig.height=5,fig.cap = "Vertical stacked barplot of LGF frequencies on chromosome 3 for type 1 samples."}
datacolumns <- names(temp[["frequency"]])
datacolumns
image(bandData, what = datacolumns[1:3], chr = 2, labels = TRUE)
```
By adding the parameter <tt>horix=TRUE</tt>, you can rotate this graph 90 degrees. For
more details about the parameters of the <tt>image</tt> method, see the manual pages
and the "gallery" vignette.

# Idiograms
We can assemble all of the single-chromosome plots into a single "idiogram" graph that
shows all chromosomes at once.


## One Data Column
The purpose of this graph is to visualize the chromosomes as well as a barplot of the
cytogenetic abnormalities in orderto observe and possibly identify patterns.
```{r makeid, fig.width=10, fig.height=8, fig.cap="Idiogram for one data columm."}
image(bandData, what = datacolumns[1], chr = "all", pal = "orange")
```

## More Data Columns
This graph allows the user to compare and contrast two or more cytogenetic events
simultaneously. Here we show loss (orange), gain (green), and fusion (purple) events
from the Type 1 samples.
```{r biid, fig.width=10, fig.height=8, fig.cap="Idiogram to contrast two data columms."}
image(bandData, what = datacolumns[1:3], chr = "all", 
      pal=c("orange", "forestgreen", "purple"), horiz=TRUE)
```

# Gallery
To see all possible visuals please go to our gallery for images.

# Appendix
```{r}
sessionInfo()

```

\name{CytobandData-class}
\alias{CytobandData-class}
\alias{CytobandData}
\alias{summary,CytobandData-method}
\title{The \code{CytobandData} Class}
\description{
  A class to represent genome-wide data measured or summarized at
  cytoband-level resoluton.
}
\usage{
CytobandData(data, info, genome = NULL)
\S4method{summary}{CytobandData}(object, \dots)
}
\arguments{
  \item{data}{ A data frame that must contain at least one
    numeric column and may contain the five cytoband location
    columns. If the latter are missing, they must be supplied by the
    \code{genome} argument.}
  \item{info}{ A data frame with \code{Label} and
    \code{Description} columns. If missing, it will be created from the
    column names in the \code{data} argument.}
  \item{genome}{ A data frame containing cytoband
    locations. Often, simply uses the \code{\link{cytobandLocations}}
    object describing build 38 of the human genome.}
  \item{object}{An object of the \code{CytobandData} class.}
  \item{\dots}{Additional arguments to the summary method; ignored.}
}
\section{Slots}{
  \describe{
    \item{\code{DATA}:}{A data frame containg five columns  (Chromosome,
      loc.start, loc.end, Band, and Stain) defining chromosomal
      locations of cytobands, along with one or more columns of
      numerical data at cytoband resolution.}
    \item{\code{INFO}:}{A data frame with two columns (Label and
      Description) that describes the columns of the \code{DATA} slot.}
  }
}
\section{Methods}{
  \describe{
    \item{summary:}{\code{signature(object = "RCytoGPS")}
      Returns a summary of the \code{DATA} slot.
    }
  }
}
\details{
  Conventional karyotyping describes chromosomal abnormalities (in a
  standardized text-based nomenclature) that are visible through a
  microsope. Karyotyping relies on a technique known as Giemsa staining,
  which creates a banding pattern along the chromosome of different
  shades of gray. The Loss-Gain-Fusion (LGF) model implemented at the
  \url{https://cytogps-app.mangograss-a1cf8a6c.westus2.azurecontainerapps.io/}
  web site converts text-based karyotypes into
  binary vectors, stored in JSON files. Our \code{\link{readLGF}}
  function parses the JSON files to create R data structures, including
  cytoband-resolution summaries of the frequencies of
  abnormalities. These summaries can be used to create
  \code{CytobandData} objects, which can be visualized using the
  \code{\link[graphics]{barplot}} and \code{\link[graphics]{image}} methods.
}
\value{
 The \code{CytobandData} conxtructor returns an object of the
 \code{CytobandData} class. The \code{summary} method returns a table,
 which is the usual return value when the summary method is applied to a
 data frame (in this case, the \code{DATA} slot).

}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Dwayne G. Tally \email{dtally110@hotmail.com}
}
\references{
  Abrams ZB, Tally DG, Coombes KR. RCytoGPS: An R Package for Analyzing
  and Visualizing Cytogenetic Data. In preparation.

  Abrams ZB, Tally DG,  Zhang L, Coombes CE, Payne PRO, Abruzzo LV,
  Coombes KR. Pattern recognition in lymphoid malignancies using CytoGPS
  and Mercator. Under review.
}
\examples{
jsonDir <- system.file("Examples/JSONfiles", package = "RCytoGPS")
temp <- readLGF(folder = jsonDir)
cytoData <- data.frame(temp[["CL"]],temp[["frequency"]])
bandData <- CytobandData(cytoData)
class(bandData)
summary(bandData)
}

\name{cytobandLocations}
\alias{cytobandLocations}
\alias{idiocolors}
\docType{data}
\title{Cytoband Locations}
\description{
  This data set contains the genomic locations of cytobands based on
  both the latest build of the human genome (GRch38) and the latest
  update to the International Standard for human Cytogenomic
  Nomenclature (ISCN). Note that the CRch38 locations are unchanged from
  build GRch37 (hg19). 
}
\usage{
data(cytobandLocations)
}
\format{A data matrix (\code{cytobandLocations}) containing 868 rows and
  5 columns. Each row contains one of the cytobands defined in the 2016
  update to ISCN nomenclature. The rownames are the standard cytoband
  names; for example, \code{1p36.33}. The columns are
  \describe{
    \item{Chromosome}{The name of the human chromosome, stored as
      \code{chr#}.}
    \item{\code{loc.start}}{The starting base position of the band.}
    \item{\code{loc.end}}{The ending base position of the band.}
    \item{\code{Band}}{The band name without the chromosome; for
      example, \code{p36.33}.}
    \item{\code{Stain}}{A factor containing the name of the Giemsa-stain
      color of the band in a karyotype image.}
  }
  Also, a vector (\code{idiocolors}) of length eight that translates the
  Giemsa stain names into appropriate colors.
}
\source{
  The starting point for these data were the cytoband locations
  downloaded from the UCSC Genome Browser. We confirmed that the data
  were unchanged between human genome builds GRCh36 (hg18), GRCh37
  (hg19), and GRCh38. However, the list of cytobands from UCSC was not
  consistent with the list of cytobands in ISCN 2016. We manually edited
  the source file to be compliant. It also matches the list of cytobands
  produced at
  \url{https://cytogps-app.mangograss-a1cf8a6c.westus2.azurecontainerapps.io/}.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Dwayne G. Tally \email{dtally110@hotmail.com}
}
\references{
  J. McGowan-Jordan, A. Simons, M. Schmid (editors).
  ISCN 2016: An International System for Human Cytogenomic Nomenclature.
  Karger Publishing, Basel, 2016.
}
\keyword{datasets}

\name{cytoband-data}
\alias{cytoband-data}
\alias{cytoData}
\docType{data}
\title{Example Cytoband Data}
\description{
  This data set contains the genomic locations of cytobands along with
  loss-gain-fusion percentages for three group of samples.
}
\usage{
data(cytoData)
}
\format{A data matrix (\code{cytobandLocations}) containing 868 rows and
  14 columns. Each row contains one of the cytobands defined in the 2016
  update to ISCN nomenclature. The rownames are the standard cytoband
  names; for example, \code{1p36.33}. The first five columns are the
  same as in the \code{\link{cytobandLocations}} data set. The remaining
  nine columns form three sets of three, recording the percentage of Loss,
  Gain, and Fusion events in three sets of samples, labeled "A", "B",
  and "C".
}
\source{
  The cytoband locations were downloaded from the UCSC Genome Browser
  and synchronized with the list of cytobands in ISCN 2016. The
  percentages were computed using tools in this package from subsets of
  the Mitelman Database of Chromosome Aberrations and Gene Fusions in
  Cancer.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Dwayne G. Tally \email{dtally110@hotmail.com}
}
\references{
  J. McGowan-Jordan, A. Simons, M. Schmid (editors).
  ISCN 2016: An International System for Human Cytogenomic Nomenclature.
  Karger Publishing, Basel, 2016.

  Mitelman, F., B. Johansson, and F. Mertens, Catalog of chromosome
  aberrations in cancer. Vol. 1. 1991: Wiley-Liss New York.
}
\keyword{datasets}

\name{readLGF}
\alias{readLGF}
\title{Extracting LGF karyotype data from JSON files }
\description{
  A function to read binary karyotype data, stored in LGF format in JSON
  files produced by the CytoGPS web site, into R for further analysis.
}
\usage{
readLGF(files = NULL, folder = NULL, verbose = TRUE)
}
\arguments{
  \item{files}{The name of the JSON file (or a character vector of
    such file names) from which you want to extract and format
    data. If \code{NULL}, then it will extract all JSON files within
    the folder path provided.}
  \item{folder}{The specified directory/folder from which the user
    wants to extract JSON files. If \code{NULL}, then the function
    will look in the current working directory.}
  \item{verbose}{A logical value; should the function keep you informed about what it is doing?}
}

\value{
  A list containing five elements:

    \item{\code{source}:}{A character vector containing the names of the
      JSON files from which data was read.}
    \item{\code{raw}:}{A list of lists, one per JSON source file. Each
      internal list contains two elements, \code{Status} and
      \code{LGF}. \code{Status} is a data frame with one row per
      karyotype in the input file, describing the results of CytoGPS
      parsing and mapping. Results can be "\code{Success}",
      "\code{Nonfixable grammar error}",
      "\code{Validation error}",
      "\code{Fixable grammar error and success}",  or
      "\code{Fixable grammar error but containing validation error}".
      \code{LGF} is a data frame where the columns are LGF-cytobands and the
      rows are clones from the successfully processed input karyotypes;
      each karyotype can have multiple clones. Entries are zero or one
      indicating the absence or presence of an abnormality.}
    \item{\code{frequency}:}{A data frame, where the rows are cytobands
      and the columns are Loss, Gain, and Fusion, with three columns per
      input file. Entries are the fraction of karyotype clones with
      that abnormality.}
    \item{\code{size}:}{An integer vector containing the total number of
      clones per input file. These values can be used to turn fractions
      back into counts.}
    \item{\code{CL}:}{A data frame with one row per cytoband detailing
      the chromosomal location and (grayscale) color of the band
      produced by Giemsa staining.}

}
\details{

  CytoGPS is an algorithm that converts conventional karyotypes from the
  standard text-based notation (the International Standard for Human
  Cytogenetic/Cytogenomic Nomenclature; ISCN) into binary vectors with
  three bits (loss, gain, or fusion) per cytoband, which we call the LGF
  model. The web site
  \url{https://cytogps-app.mangograss-a1cf8a6c.westus2.azurecontainerapps.io/}
  provides an
  implementation that allows users to upload text files containing one
  karyotype per line. It produces its output as a file in JavaScript
  Object Notation (JSON).

  The \code{readLGF} function reads and parses these JSON files and
  converts them into an R data structure. The \code{raw} component of
  this structure contains binary matrices that can serve as input to the
  Mercator package (see \code{\link[Mercator]{Mercator-class}}) for
  unsupervised analyses. The \code{frequency} component summarizes the
  fraction of input karyotype-clones with each abnormality, and can be
  visualized with other tools in the \code{RCytoGPS} package.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Dwayne G. Tally \email{dtally110@hotmail.com}
}
\seealso{
  \code{\link[Mercator]{Mercator-class}}
}
\references{
  Abrams ZB, Zhang L, Abruzzo LV, Heerema NA, Li S, Dillon T, Rodriguez R,
  Coombes KR, Payne PRO. CytoGPS: a web-enabled karyotype analysis tool
  for cytogenetics. Bioinformatics. 2019 Dec 15;35(24):5365-5366.

  Abrams ZB, Tally DG,  Zhang L, Coombes CE, Payne PRO, Abruzzo LV,
  Coombes KR. Pattern recognition in lymphoid malignancies using CytoGPS
  and Mercator. Under review.

  Abrams ZB, Tally DG, Coombes KR. RCytoGPS: An R Package for Analyzing
  and Visualizing Cytogenetic Data. In preparation.
}
\examples{
jsonDir <- system.file("Examples/JSONfiles", package = "RCytoGPS")
x <- readLGF(folder = jsonDir)

jsonFile <- dir(jsonDir, pattern = "*.json")[1]
y <- readLGF(jsonFile, jsonDir)

}
\keyword{
IO
}

\name{Chromosome-class}
\alias{Chromosome-class}
\alias{Chromosome}
\alias{image,Chromosome-method}
\title{The \code{Chromosome} Class}
\description{
  A class to represent a single chromsome in order to plot an image of
  the Giemsa-stained cytobands.
}
\usage{
Chromosome(I, res = 2500, maxbase = 250000000)
\S4method{image}{Chromosome}(x, horiz = FALSE, mai = NULL, showBandNames = FALSE, ...)
}
\arguments{
  \item{I}{A human chromosome identifier; one of the values in \code{c(1:22,"X", "Y")}.}
  \item{res}{An integer resolution; the number of pixels used to
    represent an entire chromosome.}
  \item{maxbase}{An integer representing an upper bound on the length of
    the longest chromosome, mneasured in base-pairs.}
  \item{x}{An object of the \code{Chromosome} class.}
  \item{horiz}{A logical value: should the image of the chromsome be
    oriented horizontally.}
  \item{mai}{margin inches, as in the usual graphical argument to \code{par}.}
  \item{showBandNames}{logical; shold the names of the bands be written on the plot?}
  \item{\dots}{Additional arguments to the image method; ignored.}
}
\section{Slots}{
  \describe{
    \item{\code{name}:}{A character value, typically of the form \code{"chr1"}.}
    \item{\code{label}:}{A chartacter value, typiclly of the form
      \code{"Chr 1"}.}
    \item{\code{grid}:}{An integer vector (of length \code{res})
      repesenting the base position along the chromosome.}
    \item{\code{range}:}{An integer vector of length 2 marking teh
      starting and ending position of the banded part of the chromosome,
      in bases.}
    \item{\code{stain}:}{An integer vector (of length \code{res}), where
      the values indicate the color of the Giemsa stain for that part of
      the chromosome.}
  }
}
\section{Methods}{
  \describe{
    \item{image:}{\code{signature(object = "RCytoGPS")}
      Creates an image of the chromosme, with bands colored according
      to Giemsa staining.
    }
  }
}
\details{
  Conventional karyotyping describes chromosomal abnormalities (in a
  standardized text-based nomenclature) that are visible through a
  microsope. Karyotyping relies on a technique known as Giemsa staining,
  which creates a banding pattern along the chromosome of different
  shades of gray. This class is used to plot images of individual
  chromosomes, corectly reflecting the size and color of the bands..
}
\value{
 The \code{Chromosome} constructor returns an object of the
 \code{Chromosome} class. The \code{image} method invisibly returns its
 first argument.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Dwayne G. Tally \email{dtally110@hotmail.com}
}
\references{
  Abrams ZB, Tally DG, Coombes KR. RCytoGPS: An R Package for Analyzing
  and Visualizing Cytogenetic Data. In preparation.

  Abrams ZB, Tally DG,  Zhang L, Coombes CE, Payne PRO, Abruzzo LV,
  Coombes KR. Pattern recognition in lymphoid malignancies using CytoGPS
  and Mercator. Under review.
}
\examples{
x <- Chromosome(6)
image(x)
image(x, showBandNames = TRUE)
image(x, horiz = TRUE)
}

\name{preclean}
\alias{preclean}
\title{Pre-clean Karyotypes}
\description{
  A function to clean karyotype data, by deleting known comments that do not
  adhere to the ISCN standard.
}
\usage{
preclean(x, targetColumns, dirt)
}
\arguments{
  \item{x}{A data frame containing at least one column of karyotype data.}
  \item{targetColumns}{Either a numeric vector of column indices or a
    character vector of column names.}
  \item{dirt}{A character vector containing items to delete from all karyotypes,}
}

\value{
  A data frame of te same size and with the same number of columns as
  the input data frame.
}
\details{
  The core input data worked on by the \code{RCytoGPS} are karyotypes,
  which are text strings written to conform to the ISCN standard. At
  many institutions, the cytogeneticists have developed idiosyncratic
  conventions that they use to add comments into the string. In most
  cases, these karyotypes are simply stored as text strings in a local
  database. In particular, they are not checked for synatx or grammar
  errors. By contrast, the implementation of the CytoGPS algorithm at
  the web site
  \url{https://cytogps-app.mangograss-a1cf8a6c.westus2.azurecontainerapps.io/}
  uses a formal approach with
  lexer and parser. As a result, many karyotypes are rejected by the
  system.

  The \code{preclean} function uses \code{\link[base]{gsub}} to delete a list
  of known (local) comments from all karyotypes, making it more likley
  that they will be successfully processed by the lexer and parser. We
  provide an example list derived from experience at our own
  institution.

  \emph{Implementation Note:} The \code{preclean} function removes
  strings in the order that they are contained in the \code{dirt}
  vector. So, you have to be carefully not to delete parts of a long
  phrase before trying to delete the whole phrase. For example, you
  should not remove "clonal" before removing "nonclonal". 
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com}
}
\references{
  Abrams ZB, Zhang L, Abruzzo LV, Heerema NA, Li S, Dillon T, Rodriguez R,
  Coombes KR, Payne PRO. CytoGPS: a web-enabled karyotype analysis tool
  for cytogenetics. Bioinformatics. 2019 Dec 15;35(24):5365-5366.

  Abrams ZB, Tally DG,  Zhang L, Coombes CE, Payne PRO, Abruzzo LV,
  Coombes KR. Pattern recognition in lymphoid malignancies using CytoGPS
  and Mercator. In Press.

  Abrams ZB, Tally DG, Coombes KR. RCytoGPS: An R Package for Analyzing
  and Visualizing Cytogenetic Data. In preparation.
}
\examples{
cleanDir <- system.file("PreClean", package = "RCytoGPS")
bad <- read.delim(file.path(cleanDir, "badStrings.txt"), header=FALSE)
bad <- as.vector(as.matrix(bad))
input <- read.csv(file.path(cleanDir, "startKaryotypes.csv"))
myclean <- preclean(input, 4:5, bad)
}
\keyword{
character
}

\name{CytobandData-methods}
\alias{image,CytobandData-method}
\alias{barplot,CytobandData-method}
\alias{Idiogram Graphs}
\title{Idiogram Graphs}
\description{
  The \code{image} and \code{barplot} methods of the
  \code{\link{CytobandData-class}} provide flexible displays of genome
  wide data that has been summarized at cytoband resolution.
}
\usage{
\S4method{image}{CytobandData}(x, chr, what,
         pal = palette(), nrows = 2, labels = NULL,
         horiz = FALSE, axes = chr != "all", debug = FALSE, legend = FALSE,
         sigcolumn = NA, sigcut = 0.01, alpha = 63, clip = FALSE)
\S4method{barplot}{CytobandData}(height, what, col = "blue", altcol = "#FED4C4",
                   ylab = "Percent", hline = NULL,
                   xform = function(x) x, ...)
}
\arguments{
  \item{x}{An object of the \code{\link{CytobandData-class}}, which
    combines numerical data at cytoband resolution with information on
    the chromosomal locations of the cytobands.}
  \item{chr}{The specific chromosome you want to view, typically in
    \code{c(1:22, "X", "Y")}. if you want to see all the chromosomes
    at once then you can set \code{chr = "all"}.}
  \item{what}{A vector or list containing the names of the numerical
    column(s) that you want to display from the data frame. The
    \code{barplot} method only shows a single data column at a time. The
    plots resulting from the \code{image} method change depending on
    whether you supply a vector or a list, as well as on the length. For
    more details, see the vignettes, espefcialy the image gallery.}
  \item{pal}{a charcter vector containing the colors you want to use
    for different data shown in the plot.}
  \item{horiz}{A logical value determining whether images present the
    chromosome idiograms horizontally or vertically.}
  \item{nrows}{Only used when \code{chr = "all"} to determine the number
    of rows to use to display different chromosomes. Must be an integer
    between 1 and 4.}
  \item{labels}{Only use when \code{what} is a character vector (not a
    list) and \code{chr} is mot equal to \code{"all"}. Used to label
    differnt columns of displayed data.}
  \item{axes}{Logical value; should axes be displayed?}
  \item{legend}{Logicla value; should a legend be added to the plot.}
  \item{sigcolumn}{The three parameters \code{sigcolumn},
    \code{sigcut}, and \code{alpha} are used as a set. The first names a
    numerical column in the data set used to define "significance". The
    second is a vector of cutoffs that mark levels of significance. The
    latter is a number between 0 and 255 denoting the transparency level
    assigned to the color for each significance level.}
  \item{sigcut}{See \code{sigcolumn}.}
  \item{alpha}{See \code{sigcolumn}.}
  \item{clip}{A logical value; should te length of the chromosome fill
    the device (if TRUE) or be plotted relative to the length of Chromosome
    1 (if FALSE). Currently only used when plotting two values, one on
    either side of a single chromosome.}
  \item{debug}{Logical value; should the method print out debugging
    information. Probably best to ignore.}

  \item{height}{An object of the \code{\link{CytobandData-class}}, which
    combines numerical data at cytoband resolution with information on
    the chromosomal locations of the cytobands.}
  \item{col}{a character vector containing the colors you want to use
    for different data shown in the plot.}
  \item{altcol}{Determines the color used, lternating with white, in the
    x-axis plot of all chromosomes.}
  \item{ylab}{Label for the y-axis in a barplot.}
  \item{hline}{Numeric vector of heights at which toadd a horizontal
    line.}
  \item{xform}{Function to transform the data before plotting. Defauly
    is the identity map, which does nothing.}
  \item{\dots}{Ignored.}
}
\value{
  Both the \code{image} and \code{barplot} methods invisibly return
    their first argument, and object fo the \code{CytobandData} class.
}
\author{
  Kevin R. Coombes \email{krc@silicovore.com},
  Dwayne G. Tally \email{dtally110@hotmail.com}
}
\references{
  Abrams ZB, Tally DG, Coombes KR. RCytoGPS: An R Package for Analyzing
  and Visualizing Cytogenetic Data. In preparation.

  Abrams ZB, Tally DG,  Zhang L, Coombes CE, Payne PRO, Abruzzo LV,
  Coombes KR. Pattern recognition in lymphoid malignancies using CytoGPS
  and Mercator. Under review.
}
\examples{
jsonDir <- system.file("Examples/JSONfiles", package = "RCytoGPS")
x <- readLGF(folder = jsonDir)
cytoData <- data.frame(x[["CL"]], x[["frequency"]])
bandData <- CytobandData(cytoData)
datacolumns <- names(x[["frequency"]])
barplot(bandData, what = datacolumns[1], col="forestgreen")

image(bandData, what = datacolumns[1:3], chr = 2)
image(bandData, what = datacolumns[1:3], chr = "all")
image(bandData, what = as.list(datacolumns[1:2]), chr = 2)
image(bandData, what = as.list(datacolumns[1:2]), chr = "all")
}
\keyword{graphics}
